package com.mortals.xhx.module.one.service.impl;

import com.mortals.framework.model.OrderCol;
import com.mortals.xhx.common.code.TreeTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.one.model.*;
import com.mortals.xhx.module.one.service.OneClassifyService;
import com.mortals.xhx.module.one.service.OneClassifyTopicalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.one.dao.OneTopicalDao;
import com.mortals.xhx.module.one.service.OneTopicalService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>Title: 主题表</p>
 * <p>Description: OneTopicalService service接口 </p>
 * <p>Copyright: Copyright &reg;  </p>
 *
 * @version 1.0.0
 */
@Service("oneTopicalService")
public class OneTopicalServiceImpl extends AbstractCRUDServiceImpl<OneTopicalDao, OneTopicalEntity, Long> implements OneTopicalService {

    @Autowired
    private OneClassifyTopicalService oneClassifyTopicalService;
    @Autowired
    private OneClassifyService oneClassifyService;

    @Override
    public List<Map<String, Object>> getTreeList(Integer type) {
        OneTopicalQuery oneTopicalQuery = new OneTopicalQuery().deleted(YesNoEnum.NO.getValue()).topicalType(type);
        oneTopicalQuery.setOrderColList(Arrays.asList(new OrderCol("topical_rank", OrderCol.ASCENDING)));

        List<Map<String, Object>> collect = this.find(oneTopicalQuery).stream().map(item -> {
            Map<String, Object> data = new HashMap<>();

            data.put("name", item.getTopicalName());
            data.put("id", item.getId());

            data.put("type", TreeTypeEnum.TOPIC.getValue());
            data.put("icon", TreeTypeEnum.TOPIC.getDesc());
            data.put("isLeaf", true);

            OneClassifyTopicalQuery oneClassifyTopicalQuery = new OneClassifyTopicalQuery().deleted(YesNoEnum.NO.getValue()).topicalId(item.getId());
            oneClassifyTopicalQuery.setOrderColList(Arrays.asList(new OrderCol("classify_rank", OrderCol.ASCENDING)));

            List<Map<String, Object>> children = oneClassifyTopicalService.find(oneClassifyTopicalQuery)
                    .stream().map(classify -> {
                        OneClassifyEntity oneClassifyEntity = oneClassifyService.get(classify.getClassifyId());
                        Map<String, Object> map = new HashMap<>();
                        map.put("id", classify.getId());
                        map.put("name", oneClassifyEntity==null?"":oneClassifyEntity.getClassifyName());
                        map.put("isLeaf", true);
                        map.put("type", TreeTypeEnum.CLASSIFY.getValue());
                        map.put("icon", TreeTypeEnum.CLASSIFY.getDesc());
                        return map;
                    }).collect(Collectors.toList());

            if (children.size() > 0) {
                data.put("isLeaf", false);
                data.put("children", children);
            }
            return data;

        }).collect(Collectors.toList());

        return collect;
    }
}